##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: __init__.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""

import os
import sys

# this code is temporarily here. It will be moved to SQLObject later.
import sqlobject

class SODateCol(sqlobject.col.SOCol):

    def _mysqlType(self):
        return 'DATE'

    def _postgresType(self):
        return 'DATE'

class DateCol(sqlobject.col.Col):

    baseClass = SODateCol

sqlobject.col.DateCol = DateCol

def package_home(gdict):
    filename = gdict["__file__"]
    return os.path.dirname(filename)

class File:

    _v_last_read = 0
    _v_text = None

    def __init__(self, filename, _prefix=None):
        path = self.get_path_from_prefix(_prefix)
        self.filename = os.path.join(path, filename)
        if not os.path.isfile(self.filename):
            raise ValueError("No such file", self.filename)

    def get_path_from_prefix(self, _prefix):
        if isinstance(_prefix, str):
            path = _prefix
        else:
            if _prefix is None:
                _prefix = sys._getframe(2).f_globals
            path = package_home(_prefix)
        return path

    def _cook_check(self):
        if self._v_last_read and not __debug__:
            return
        __traceback_info__ = self.filename
        try:
            mtime = os.path.getmtime(self.filename)
        except OSError:
            mtime = 0
        if mtime == self._v_last_read:
            return
        f = open(self.filename, "rb")
        try:
            text = f.read()
        except:
            f.close()
            raise
        f.close()
        self._v_text = text
        self._v_last_read = mtime

    def __str__(self):
        self._cook_check()
        return self._v_text

from project import Project
from sample import Sample, AdditionalSampleData
from subjecttype import SubjectType
from sampletype import SampleType
from container import Container
from container_location import ContainerLocation
from containertype import ContainerType
from location import Location
from location_project import LocationProject
from aliquot import Aliquot
from technician import Technician
from technician_project import TechnicianProject
from principal import Principal
from units import ConcentrationUnit, VolumeUnit, WeightUnit
from security.principal import SecurityPrincipal
from security.role import SecurityRole
from schema.fields import SchemaField
from sampletypeschema import SampleTypeSchema
from sampletype_project import SampleTypeProject

__all__ = ('Project', 'Principal',
           'Technician', 'TechnicianProject',
           'Location', 'LocationProject', 'SubjectType',
           'SampleType', 'SampleTypeProject', 'Sample',
           'ContainerType', 'Container', 'ContainerLocation',
           'ConcentrationUnit', 'VolumeUnit', 'WeightUnit',
           'Aliquot', 'SecurityPrincipal', 'SecurityRole',
           'SampleTypeSchema', 'SchemaField',
           'AdditionalSampleData')
