##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: admin.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""

__metaclass__ = type

from zope.app import zapi
from zope.app.publisher.browser import BrowserView
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile

from sip.browser import api

class AdminView(BrowserView):

    def __init__(self, context, request):
        super(AdminView, self).__init__(context, request)
        self.setupViews()

    def setupViews(self):
        context = self.context
        request = self.request
        self.views = {}
        self.forms = {}
        project = zapi.getView(context,
                               'edit.html', request)
        self.views['project'] = NextURLProxy(project, self)
        self.forms['project'] = project.generated_form

        locations = zapi.getView(context,
                                 'locations.html', request)
        self.views['locations'] = NextURLProxy(locations, self)
        self.forms['locations'] = locations.generated_form
        technicians = zapi.getView(context,
                                   'technicians.html', request)
        self.views['technicians'] = NextURLProxy(technicians, self)
        self.forms['technicians'] = technicians.generated_form
        sampletypes = zapi.getView(context,
                                   'sampletypes.html', request)
        self.views['sampletypes'] = NextURLProxy(sampletypes, self)
        self.forms['sampletypes'] = sampletypes.generated_form
        avail_st = zapi.getView(context,
                                'avail_st.html', request)
        self.views['avail_st'] = NextURLProxy(avail_st, self)
        self.forms['avail_st'] = avail_st.generated_form

    def update(self):
        view = None
        if 'EDIT_PROJECT' in self.request:
            view = 'project'
        elif 'MODIFY_SAMPLE_TYPE' in self.request:
            view = self.views['avail_st'].getView()
            st_id = view.sample_type_widget.getInputValue()
            schema = api.getSchemaFor(st_id, self.context)
            url = zapi.getView(schema, 'absolute_url', self.request)()
            self.request.response.redirect(url + '/@@editschema.html')
            return
        elif 'ASSIGN_SAMPLE_TYPES' in self.request:
            view = 'sampletypes'
        elif 'ASSIGN_LOCATIONS' in self.request:
            view = 'locations'
        elif 'ASSIGN_TECHNICIANS' in self.request:
            view = 'technicians'
        if view:
            view = self.views[view].getView()
            if not view.update_status:
                status = view.update()
                self.setupViews()
                return status

    admin = ViewPageTemplateFile('admin.pt')

    def hasAssignedLocations(self):
        return len(self.context.valid_locations)

    def hasAssignedTechnicians(self):
        return len(self.context.valid_technicians)

    def hasSampleTypes(self):
        return len(zapi.traverse(self.context, 'sampletypes'))

    def nextURL(self):
        return (str(zapi.getView(self.context, "absolute_url", self.request))
                + '/@@admin_project.html')

class NextURLProxy:

    def __init__(self, context, target, target_methods=('nextURL', 'update')):
        self.__context = context
        self.__target = target
        self.__target_methods = target_methods

    def getView(self):
        return self.__context

    def __getattribute__(self, name):
        if name in ('_NextURLProxy__context',
                    '_NextURLProxy__target',
                    '_NextURLProxy__target_methods',
                    'getView'):
            return object.__getattribute__(self, name)
        if name in self.__target_methods:
            return getattr(self.__target, name)
        return getattr(self.__context, name)

    def __setattr__(self, name, value):
        if name in ('_NextURLProxy__context',
                    '_NextURLProxy__target',
                    '_NextURLProxy__target_methods',
                    'getView'):
            object.__setattr__(self, name, value)
        else:
            setattr(self.__context, name, value)
