##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: aliquot_list.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""

from zope.proxy import removeAllProxies
from zope.app.publisher.browser import BrowserView
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from zope.app import zapi
from sip.interfaces.adapter import ISampleWithAliquots,\
     ISampleWithOriginAliquots
from zope.security.proxy import removeSecurityProxy
from sip.browser import api

class AliquotListing(BrowserView):
    """Aliquot Listing For Sample"""

    def listAliquotInfo(self):

        extract = self._extractInfo
        sample = removeSecurityProxy(self.context)
        adapted = ISampleWithAliquots(sample)
        aliquots = adapted.aliquots.values()
        info = map(extract, aliquots)

        return info

    def _extractInfo(self, aliquot):

        info = {}
        root = api.getRoot(self.context)
        root_url = str(zapi.getView(root, 'absolute_url', self.request))
        info['id'] = aliquot.id
        info['label'] = aliquot.label
        sample = removeAllProxies(self.context)
        info['aliquot_url'] = ('%s/projects/sip.Project.%d/aliquots/'
                               'sip.Aliquot.%d' %
                               (root_url,
                                sample.project_id,
                                aliquot.id) )
        info['url'] = ('%s/projects'
                       '/sip.Project.%d/samples/sip.Sample.%s'
                       '?aliquot_id=%s&SAMPLE_EDIT_FORM=1' %
                       (root_url,
                        sample.project_id,
                        sample.id,
                        aliquot.id))
        info['concentration'] = aliquot.getConcentrationString()
        info['weight'] = aliquot.getWeightString()
        info['volume'] = aliquot.getVolumeString()
        info['location'] = aliquot.getLocation()
        info['is_low'] = aliquot.is_low and 'Yes' or 'No'
        if aliquot.aliquot_count is None:
            info['aliquot_count'] = '&nbsp;'
        else:
            info['aliquot_count'] = aliquot.aliquot_count
        info['depleted'] = aliquot.depleted and 'Yes' or 'No'
        info['container_type'] = aliquot.getContainerType() or '&nbsp;'
        info['comment'] = aliquot.comment
        return info

    index = ViewPageTemplateFile('aliquot_list.pt')

class OriginAliquotListing(AliquotListing):

    def listAliquotInfo(self):

        extract = self._extractInfo
        sample = removeSecurityProxy(self.context)
        adapted = ISampleWithOriginAliquots(sample)
        aliquots = adapted.origin_aliquots.values()
        info = map(extract, aliquots)

        return info

    def _extractInfo(self, aliquot):

        info = super(OriginAliquotListing, self)._extractInfo(aliquot)
        aliquot = removeAllProxies(aliquot)
        sample = aliquot.sample_
        info['sample'] = sample.name
        root = api.getRoot(self.context)
        root_url = str(zapi.getView(root, 'absolute_url', self.request))
        info['sample_url'] = ('%s/projects'
                              '/sip.Project.%d/samples/sip.Sample.%d' %
                              (root_url, sample.project_id, sample.id))
        return info
