##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: container.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""

__metaclass__ = type

from zope.app import zapi
from zope.event import notify
from zope.app.event.objectevent import ObjectCreatedEvent
from zope.app.container.interfaces import IContainer
from sip.interfaces.location import ILocation
from sip.browser import api
from sip.shipment.interfaces import IShipment

class AddContainer:

    _rows = None
    _cols = None
    _name = None

    def __init__(self, context, request):
        # XXX This is bad, as we are changing the
        # rug under our feet. The solution is to
        # provide a custom IAdding to ILocation, or
        # to change zope's IAdding to adapt to IContainer.
        container = IContainer(context.context)
        if container is not context.context:
            context.context = container
        super(AddContainer, self).__init__(context, request)

    def create(self, *args, **kw):
        parent = zapi.getParent(self.context)
        if ILocation.providedBy(parent) or IShipment.providedBy(parent):
            parent = parent.getRootContainer()
            if parent is None:
                raise ValueError, "Root container not found in location"
        parentid = parent.id
        kw['parent_id'] = parentid
        root = api.getRoot(parent)
        ct = root['containertypes']['sip.ContainerType.%s' % kw['type_id']]
        typename = ct.name
        name = kw['name'] or typename
        if not self._rows:
            if kw['rows']:
                self._rows = kw['rows']
            else:
                self._rows = 1
            if kw['columns']:
                self._cols = kw['columns']
            else:
                self._cols = 1
            self._name = kw['name']
            del kw['rows']
            del kw['columns']

        factory = api.factory('sip.Container', context=self.context)

        for r in range(1, self._rows+1):
            for c in range(1, self._cols + 1):
                # SdS: Blame on me if this piece of code confusing
                # but IMHO its better than two 'if' and three 'for'
                #
                # If _rows == 1, return 'c', if _cols == 1, 'r'
                # else, 'rXc'
                postfix = ("%s" % ((self._rows == 1 and
                                    c or
                                    self._cols == 1 and
                                    r) or
                                   "%sx%s" % (r, c)))
                if not (self._rows == 1 and self._cols == 1):
                    kw['name'] = "%s-%s" % (name, postfix)
                child = factory(**kw)
                #api.setLocation(self.context, child, location)
                notify(ObjectCreatedEvent(child))

        return child

    def createAndAdd(self, data):
        content = super(AddContainer, self).createAndAdd(data)

    def nextURL(self):
        url = str(zapi.getView(zapi.getParent(self.context),
                               'absolute_url', self.request))
        if IShipment.providedBy(zapi.getParent(self.context)):
            return url
        return url + '/@@close.html'
