##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: principal.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""
__metaclass__ = type

from zope.app import zapi
from zope.app.publisher.browser import BrowserView
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile

from sip.browser import api
from sip.browser.admin import NextURLProxy
from sip.interfaces.adapter import IPrincipalRolesHelper

class PrincipalManagementView(BrowserView):

    update_status = None

    def __init__(self, context, request):
        super(PrincipalManagementView, self).__init__(context, request)
        self.views = {}
        self.forms = {}
        root = api.getRoot(context)
        principals = root['principals']
        adding = zapi.getView(principals, '+', request)
        adding = NextURLProxy(adding, self)
        adding = api.contained(adding, context)
        principal_add = zapi.getView(adding, 'add_principal.html', request)
        self.views['principal'] = NextURLProxy(principal_add, self)
        self.forms['principal'] = principal_add.generated_form
        princ_roles = zapi.getView(context,
                                   'PrincipalRoles.html', request)
        self.views['princ_roles'] = NextURLProxy(princ_roles, self)
        self.forms['princ_roles'] = princ_roles.index
        request.form['Filter'] = 'Filter'
        adapter = IPrincipalRolesHelper(self.context)
        request.form['principals'] = adapter.availablePrincipals()
        request.form['roles'] = adapter.availableRoles()

    def update(self):
        view = None
        if 'MANAGE_USERS' in self.request:
            # Filter out roles higher than or equal ProjectManager
            adapter = IPrincipalRolesHelper(self.context)
            roles = adapter.availableRoles()
            for k, v in self.request.items():
                if k.startswith('grid'):
                    role = '.'.join(k.split('.')[1:3])
                    if role not in roles:
                        del self.request.form[k]
            view = 'princ_roles'
        elif 'ADD_PRINCIPAL' in self.request:
            view = 'principal'
        if view:
            view = self.views[view].getView()
            if self.update_status:
                return self.update_status
            status = view.update()
            self.update_status = status
            return status

    principal = ViewPageTemplateFile('principal.pt')

    def nextURL(self):
        return (str(zapi.getView(self.context, "absolute_url", self.request))
                + '/manage_principals.html')

class AddPrincipal:

    def create(self, *args, **kw):
        """Do the actual instantiation."""
        del kw['confirm_pass']
        return super(AddPrincipal, self).create(*args, **kw)
