##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: project_list.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""

from zope.app import zapi
from sip.browser import api
from zope.app.publisher.browser import BrowserView
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from zope.app.container.interfaces import IReadContainer

class ProjectListing(BrowserView):
    """ Project Listing view for SIP App Root """

    def canAdminApp(self):
        return api.checkSIPPermission(self.context, 'sip.ManageApplication')

    def listContentInfo(self):
        request = self.request
        info = {}
        if 'projects' in self.context.keys():
            context = zapi.traverse(self.context, 'projects')
            info = map(self._extractContentInfo,
            IReadContainer(context).items())
        return info


    def _extractContentInfo(self, item):
        request = self.request

        id, obj = item
        info = {}
        info['id'] = info['cb_id'] = id
        info['object'] = obj
        info['has_stypes'] = len(zapi.traverse(obj, 'sampletypes'))
        info['has_locs'] = len(obj.valid_locations)

        info['url'] = "projects/%s" % id

        try:
            info['title'] = obj.name
        except:
            info['title'] = '<no title>'

        isAdmin = self.canAdminApp()

        info['can_admin'] = isAdmin or api.checkSIPPermission(obj,
                                                           'sip.ManageProject')
        info['can_report'] = isAdmin or api.checkSIPPermission(obj,
                                                             'sip.ViewProject')
        info['can_add'] = isAdmin or api.checkSIPPermission(obj,
                                                               'sip.AddSample')

        return info

    index = ViewPageTemplateFile('project_list.pt')
