##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: sample_list.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""

from zope.app.publisher.browser import BrowserView
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from zope.app import zapi
from zope.app.container.interfaces import IReadContainer
from sip.browser import api

class SampleListing(BrowserView):
    '''Sample Listing View for a project'''

    def canEditSample(self):
        return api.checkSIPPermission(self.context, 'sip.EditSample')

    def canAddAliquot(self):
        return api.checkSIPPermission(self.context, 'sip.AddAliquot')

    def canAddSample(self):
        return api.checkSIPPermission(self.context, 'sip.AddSample')

    def listContentInfo(self):
        return self._listContentInfo()

    def _listContentInfo(self):
        info = {}
        context = IReadContainer(self.context)
        if 'samples' in context.keys():
            samples = zapi.traverse(context, 'samples')
            samples = IReadContainer(samples)
            info = map(self._extractContentInfo, samples.items())
        return info

    def _extractContentInfo(self, item):
        sampletypes = zapi.traverse(self.context, 'sampletypes')
        id, obj = item
        type = sampletypes['sip.SampleType.%d' % obj.type_id]
        info = {}
        info['id'] = info['cb_id'] = id
        sample_id = id.split('.')[-1]
        info['sample_id'] = sample_id
        info['object'] = obj
        info['type'] = type.name
        info['subject_type'] = api.getSubjectType(self.context,
                                                  obj.subject_type_id)
        info['subject_id'] = obj.subject_id
        info['date_generated'] = obj.date_generated
        info['url'] = "samples/%s" % id

        try:
            info['title'] = obj.name
        except:
            info['title'] = '<no title>'

        return info

    def hasSampleTypes(self):
        project = self.context
        st = zapi.traverse(project, 'sampletypes')
        st = IReadContainer(st)
        return len(st)

    index = ViewPageTemplateFile('sample_list.pt')
