##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: tree_adapters.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""

from zope.interface import implements

from zope.app import zapi
from zope.app.tree.interfaces import IChildObjects
from zope.app.container.interfaces import IReadContainer
from zope.proxy import removeAllProxies

from sip.interfaces.location import ILocation
from sip.interfaces.container import IContainer
from sip.shipment.interfaces import IShipment, IOutgoingShipmentState
from sip.interfaces.source import ILocationProjectSource
from sip.browser import api

class BaseChildObjects:
    def __init__(self, context):
        self.context = context

    def hasChildren(self):
        children = IReadContainer(self.context)
        values = children.values()
        for obj in values:
            if (IContainer.providedBy(obj) or
                ILocation.providedBy(obj) or
                IShipment.providedBy(obj) or
                IOutgoingShipmentState.providedBy(obj)):
                return True
        return False

class LocationProjectSourceChildObjects(BaseChildObjects):
    implements(IChildObjects)
    __used_for__ = ILocationProjectSource

    def getChildObjects(self):
        project = api.getProject(self.context)
        locations = zapi.traverse(project, 'locations')
        children = IReadContainer(locations)
        values = list(children.values())
        shipments = zapi.traverse(project, 'shipments/outgoing/incomplete')
        values.append(shipments)
        return values

class ContainerChildObjects(BaseChildObjects):
    implements(IChildObjects)
    __used_for__ = IContainer

    def getChildObjects(self):
        children = IReadContainer(self.context)
        return children.values()

class LocationChildObjects(ContainerChildObjects):
    implements(IChildObjects)
    __used_for__ = ILocation

class ShipmentChildObjects(ContainerChildObjects):
    implements(IChildObjects)
    __used_for__ = IShipment
