##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: widget.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""

__metaclass__ = type

from zope.app import zapi
from zope.app.form.browser.widget import renderElement
from zope.app.form.browser import BrowserWidget, TextAreaWidget, \
     TextWidget, IntWidget, DatetimeWidget, TextWidget
from zope.app.form.browser.widget import SimpleInputWidget
from zope.app.form.browser.itemswidgets import ItemsEditWidgetBase
from zope.app.form.browser import MultiSelectWidget, ChoiceInputWidget
from zope.app.form import CustomWidgetFactory
from zope.interface import implements
from sip.interfaces.fields import IROSubjectTypeWidget

class InAndOutWidget(MultiSelectWidget):

    #_msg_no_value = _msg_missing_multiple_value_edit
    size = 8

    def renderValue(self, value):
        cssClass = self.cssClass
        contents = ['<table>','<tr>','<td>']
        if value == self.context.missing_value:
            values = ()
        else:
            values = list(value)

        i = 0
        j = 0
        in_rendered_items = []
        out_rendered_items = []
        for term in self.context.value_type.vocabulary:
            item_text = self.textForValue(term)

            if term.value in values:
                rendered_item = self.renderSelectedItem(i,
                    item_text,
                    term.token,
                    self.name,
                    cssClass)
                i += 1
                in_rendered_items.append(rendered_item)
            else:
                rendered_item = self.renderItem(j,
                    item_text,
                    term.token,
                    self.name,
                    cssClass)
                j += 1
                out_rendered_items.append(rendered_item)
        name = self.name.split('.')[1]
        optname = name + '_options'
        onclick = "inout_moveKeywords('%s','%s','%s');"
        contents.append(renderElement(self.tag,
                        name=optname,
                        multiple=None,
                        size=self.size,
                        contents='\n'.join(out_rendered_items),
                        extra=' onDblClick="%s" id="%s"'% (onclick % (optname,
                            name, name), optname)))
        button = ('<input type="button" class="context"'
                 ' value="%s" onClick="%s"/>')

        contents.extend(['</td>','<td>',
            button % ('&gt;&gt;', onclick % (optname, name, name)),
            '<br />',
            button % ('&lt;&lt;', onclick % (name, optname, name)),
            '</td>','<td>'])
        contents.append(renderElement(self.tag,
                        name=self.name,
                        multiple=None,
                        size=self.size,
                        contents='\n'.join(in_rendered_items),
                        extra=' onDblClick="%s" id="%s" '%  (onclick % (
                            name,
                            optname,
                            name), name)))
        contents.extend(['</td>','</tr>','</table>'])
        return self._div(self.cssClass, "\n".join(contents),
            id=self.name)

class TALESExpressionWidget(TextWidget):

    displayWidth = 40

class CommitMessageWidget(TextAreaWidget):

    width = 35
    height = 4

    def __init__(self, context, request):
        super(CommitMessageWidget, self).__init__(context, request)

    def __call__(self):
        self.setRenderedValue(self._missing)
        return super(CommitMessageWidget, self).__call__()

class HiddenWidget(SimpleInputWidget):

    label = None

    def __call__(self):
        return self.hidden()


class BRWidget(BrowserWidget):

    def __call__(self):
        return renderElement( 'div',
                              cssClass = 'break',
                              contents = " ",
                              extra = self.getValue('extra'))

    def row(self):
        return self()

class ROTextWidget(TextWidget):

    def __init__(self, *args):
        super(TextWidget, self).__init__(*args)

    def __call__(self):
        value = getattr(self, 'value', None)
        if value is not None:
            self._data = self.value
        extra = self.extra
        if extra.lower().find('readonly') == -1:
            self.extra += ' readonly'
        return super(ROTextWidget, self).__call__()

class ROIntWidget(IntWidget):

    def __call__(self):
        value = getattr(self, 'value', None)
        if value is not None:
            self._data = self.value
        extra = self.extra
        if extra.lower().find('readonly') == -1:
            self.extra += ' readonly'
        return super(ROIntWidget, self).__call__()

class RODatetimeWidget(DatetimeWidget):

    def __call__(self):
        value = getattr(self, 'value', None)
        if value is not None:
            self._data = self.value
        extra = self.extra
        if extra.lower().find('readonly') == -1:
            self.extra += ' readonly'
        return super(RODatetimeWidget, self).__call__()


class WidgetProxy:

    def __init__(self, context, callable):
        self.__context = context
        self.__callable = callable

    def getWidget(self):
        return self.__context

    def getCallable(self):
        return self.__callable

    def __getattr__(self, name):
        if name in ('_WidgetProxy__context',
                    '_WidgetProxy__callable',
                    'row',):
            return object.__getattribute__(self, name)
        return getattr(self.__context, name)

    def __setattr__(self, name, value):
        if name in ('_WidgetProxy__context',
                    '_WidgetProxy__callable',
                    'row',):
            object.__setattr__(self, name, value)
        else:
            setattr(self.__context, name, value)

    def __call__(self):
        widget = self.getWidget()
        result = widget()
        callable = self.getCallable()
        return callable(widget.name, result)

    def row(self):
        if self._error:
            return '<div class="%s">%s</div><div class="field">%s</div>' \
                '<div class="error">%s</div>' % (self.labelClass(),
                                                 self.label(), self(),
                                                 self.error())
        else:
            return '<div class="%s">%s</div><div class="field">%s</div>' % (
                self.labelClass(), self.label(), self()
                )

class JSHelperRenderer:

    def __init__(self, context):
        self.context = context

    def __call__(self, name, result):
        context = self.context
        result = """%s<a href='%s' %s><img src='%s' alt='%s'
                         title='%s' class='%s'/>
                      </a>""" % (result,
                                 context.img_url, context.img_extra,
                                 context.img_src, context.img_alt,
                                 context.img_title, context.img_class)
        return result % {'widget_name':name}

attrs = ('img_url',
         'img_extra',
         'img_src',
         'img_alt',
         'img_title',
         'img_class',)

class JSHelperWidget(CustomWidgetFactory):

    def __init__(self, *args, **kw):
        for k in attrs:
            setattr(self, k, kw.get(k, ''))
            if k in kw: del kw[k]
        super(JSHelperWidget, self).__init__(*args, **kw)

    def __call__(self, context, request):
        instance = super(JSHelperWidget, self).__call__(context, request)
        return WidgetProxy(instance, JSHelperRenderer(self))

def ROSubjectTypeWidget(field, request):
    return zapi.getMultiView((field, field.vocabulary), request,
            IROSubjectTypeWidget)

class ROSubjectTypeEditWidget(ItemsEditWidgetBase):

    tag = 'input'
    implements(IROSubjectTypeWidget)

    def renderValue(self, value):
        if value:
            term = self.vocabulary.getTermByToken(value)
            self._data = term.value
        else:
            self._data = None
        value = self._toFormValue(value)
        return str(value) + self.hidden()
