##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: container.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""

__metaclass__ = type

from sqlobject import *

from sqlos import SQLOS
from sip.interfaces.container import IContainer, \
     IContainerSchema, IContainerAddSchema
from sip.interfaces.location import ILocation
from sip.shipment.interfaces import IShipment

from zope.interface import implements
from zope.app.container.contained import Contained
from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent

class Container(SQLOS, Contained):

    implements(IContainer, IContainerSchema, IContainerAddSchema)

    _idName='container_id'
    _table='container'
    _columns=[
        StringCol('name', length=255, notNull=1),
        ForeignKey('Container', name='parent_id',
                   default=0, cascade=False),
        ForeignKey('ContainerType', name='type_id',
                   default=None, cascade=False),
        StringCol('commit_message', notNull=1),
        ]
    classNames = ['sip.Container', 'sip.Aliquot']

    def move_to(self, new_parent):
        # XXX This should be an adapter!
        # If we write this as an adapter, then we
        # can get rid of the Contained base class.
        if ILocation.providedBy(new_parent)\
                or IShipment.providedBy(new_parent):
            new_parent = new_parent.getRootContainer()
            if new_parent is None:
                raise ValueError, "Root container not found in location"
        elif not IContainer.providedBy(new_parent):
            raise ValueError, "Invalid parent"
        self.parent_id = new_parent.id
        notify(ObjectModifiedEvent(self))
