##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: common.py,v 1.2 2005/10/21 00:08:12 dreamcatcher Exp $
"""

__metaclass__ = type

import re
from tempfile import TemporaryFile

from zope.interface import implements
from sip.csv.utils import read_csv, Chunk
from sip.interfaces.csv import IMassager
from sip.browser import api
from sip.csv.tb import MassagerTraceback

section_match = r"""section: *([a-zA-Z0-9_.]+)"""
class Massager:

    implements(IMassager)

    def __init__(self, filename):
        self.filename = filename
        self.context = None

    def __call__(self, context, data):
        self.out = Chunk()
        self.context = context
        d_lines = data.split('\n')
        d_lines = [l.strip() for l in d_lines]
        parts = {}
        section_name = None
        for line in d_lines:
            if ((line.startswith('#')
                 or line.startswith('"#'))
                and 'section:' in line):
                match = re.search(section_match, line)
                section_name = match.group(1).strip()
                parts[section_name] = []
            if (line and
                not line.startswith('#') and
                not line.startswith('"#')
                and section_name is not None):
                parts[section_name].append(line)
        return self.write(parts)

    def write(self, parts):
        for k, v in parts.items():
            tmp = TemporaryFile()
            tmp.write('\n'.join(v))
            tmp.seek(0)
            warnings = []
            read_csv(tmp, k, self.out, warnings)
            self.out.exceptions[k] = []
            tmp.close()
        return self.process()

    def process(self):
        data = Chunk()
        orig = self.out
        data.exceptions = orig.exceptions
        for name, table in orig.items():
            exceptions = data.exceptions[name]
            data[name] = new_table = table.copy()
            delete = []
            lookup_cache = {}
            name_map = {}
            for k in table.keys():
                # Utilities are looked up using
                # the 'filename:section:field'
                field_name = '%s:%s:%s' % (self.filename, table.name(), k)
                util = api.getLookupUtility(field_name)
                if util is not None:
                    # We get the new field name
                    # to be used
                    name_map[k] = util.newName()
                    lookup_cache[field_name] = util
            for k, v in name_map.items():
                # Remove the old column, initialize
                # a new column with None
                del new_table[k]
                if not v in new_table.keys():
                    new_table[v] = None
            for index, row in enumerate(table):
                __traceback_supplement__ = (MassagerTraceback,
                                            row, table.name(), index,
                                            self.filename, table)
                for k, v in row.items():
                    field_name = '%s:%s:%s' % (self.filename, table.name(), k)
                    util = lookup_cache.get(field_name)
                    if util is None:
                        continue
                    new_field = name_map.get(k)
                    try:
                        new_value = util.lookup(self.context, v)
                    except ValueError, msg:
                        exceptions.append((table.name(), k, v, str(msg)))
                        delete.append(index)
                    else:
                        new_table.update(index, new_field, new_value)
            if delete:
                delete.reverse()
                last = None
                for index in delete:
                    if index == last:
                        continue
                    new_table.delete(index)
                    last = index
        return data
