##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: metadirectives.py,v 1.2 2005/10/21 00:08:12 dreamcatcher Exp $
"""
from zope.configuration.fields import GlobalObject
from zope.interface import Interface
from zope.schema import TextLine
from sip.interfaces.csv import IProcessor, ITempStorage, IMassager

class IBaseCSVComponentDirective(Interface):
    """This directive registers a component for a CSV file."""

    filename = TextLine(
        title=u"Filename",
        description=(u"This is the filename for the CSV file "
                     u"this component will be bound to."),
        required=True)

    component = GlobalObject(
        title=u"Component",
        description=(u"Specifies the component that will act "
                     u"upon the given file."),
        required=True)

class IProvideProcessorDirective(IBaseCSVComponentDirective):
    """Provides a processor component"""

    interface = GlobalObject(
        title=u"The interface this component provides.",
        description=(u"Components provide an interface, which "
                     u"is then used to register and query through "
                     u"the utility service."),
        required=True,
        default=IProcessor,
        )

class IProvideStorageDirective(IBaseCSVComponentDirective):
    """Provides a storage component"""

    interface = GlobalObject(
        title=u"The interface this component provides.",
        description=(u"Components provide an interface, which "
                     u"is then used to register and query through "
                     u"the utility service."),
        required=True,
        default=ITempStorage,
        )

class IProvideMassagerDirective(IBaseCSVComponentDirective):
    """Provides a massager component"""

    interface = GlobalObject(
        title=u"The interface this component provides.",
        description=(u"Components provide an interface, which "
                     u"is then used to register and query through "
                     u"the utility service."),
        required=True,
        default=IMassager,
        )
