##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: common.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

__metaclass__ = type

from cStringIO import StringIO
from transaction import get_transaction

from zope.interface import implements

from sip.browser import api
from sip.csv.storage.exceptions import StorageError
from sip.interfaces.csv import IProcessor

class Processor:
    """Base Processor Class"""

    implements(IProcessor)

    def __init__(self, filename):
        self.filename = filename

    def load_data(self, context=None, data=None,
                  tid=None, request=None):
        if data is None and tid is None:
            raise ValueError, 'No data and no transaction id supplied'
        if request is None:
            raise ValueError, 'Need a request to proceed'
        if data is None:
            # No data was provided, but a transaction id.
            # We are probably resuming a transaction.
            data = self.storage.load(context=context,
                                     tid=tid)
        else:
            # If data was passed in, we need to massage it
            data = self.massager(context=context, data=data)
        return data

    def __call__(self, context=None, dry_run=True, data=None,
                 tid=None, request=None):
        self.log = StringIO()
        self.storage = api.lookupStorage(self.filename)
        self.massager = api.lookupMassager(self.filename)
        data = self.load_data(context=context, data=data,
                              tid=tid, request=request)
        if tid is None:
            # If no transaction id, create one
            tid = api.createTID(self.filename)
        # Do stuff, return log, abort if dry_run
        ok = False
        massager_failure = False
        try:
            print >> self.log, 'Started processing #%s.' % tid
            for section, table in data.items():
                print >> self.log, 'Section: %s (%s)' % (section, len(table))
            for table, exceptions in data.exceptions.items():
                if not exceptions: continue
                print >> self.log, ('The following failures occurred '
                                    'while trying to process data from '
                                    'section %s:' % table)
                massager_failure = True

                for exc in exceptions:
                    print >> self.log, str(exc)
            stats = self.process(context, data, request)
            errors = stats['errors']
            del stats['errors']
            print >> self.log, 'Finished processing #%s.' % tid
            for section, count in stats.items():
                print >> self.log, 'Section: %s (%s)' % (section, count)
            ok = not errors
        finally:
            if dry_run:
                # We call abort here to undo all changes that may
                # have been done.
                get_transaction().abort()
                # Storage's store should make sure that
                # it only calls commit on it's own transaction
                # if needed so.
                if ok and not massager_failure:
                    self.storage.store(context, data, tid)
                else:
                    # the user should not be able to complete csv if
                    # any error occurs
                    tid = None
            elif ok:
                # Should only expire if OK is set to true,
                # otherwise we may need to re-run the process
                # so the data is kept around.
                try:
                    self.storage.expire(context, tid)
                except StorageError:
                    # It is safe to ignore this error,
                    # at least for now.
                    pass
                tid = None
        return self.log.getvalue(), tid

    def process(self, context, data, request):
        return {}
