##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: descriptors.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""

__metaclass__ = type

from zope.security.checker import selectChecker
from zope.app.container._zope_app_container_contained import getProxiedObject \
     as containerProxiedObject
from sip.browser import api

class FakePropertyDescriptor:
    """ A descriptor that aliases one property to another """

    def __init__(self, name, attr_name):
        self.name = name
        self.attr_name = attr_name

    def __get__(self, obj, objtype=None):
        return getattr(obj, self.attr_name)

    def __set__(self, obj, value):
        setattr(obj, self.attr_name, value)

    def __delete__(self, obj):
        delattr(obj, self.attr_name)
        delattr(obj, self.name)

class ProjectRelationDescriptor:

    def __init__(self, className, columnName):
        self.className = className
        self.columnName = columnName

    def __get__(self, obj, objtype=None):
        factory = api.factory(self.className, context=obj)
        project_id = obj.id
        items = list(factory.select(clause="project_id = %s" % project_id))
        return [getattr(p, self.columnName) for p in items]

    def __set__(self, obj, value):
        factory = api.factory(self.className, context=obj)
        project_id = obj.id
        items = list(factory.select(clause="project_id = %s" % project_id))
        existing = [getattr(p, self.columnName) for p in items]
        for item in items:
            if getattr(item, self.columnName) not in value:
                item.destroySelf()
        for v in value:
            if v not in existing:
                kw = {}
                kw['project_id'] = project_id
                kw[self.columnName] = v
                factory(**kw)

    def __delete__(self, obj):
        raise AttributeError, "Can't delete attribute"

class BoundCallableDescriptor:

    def __init__(self, callable, call=False):
        self.callable = callable
        self.call = call

    def __get__(self, obj, objtype=None):
        def callablefunc(*args):
            return self.callable(obj, *args)
        if self.call:
            return callablefunc()
        return callablefunc

    def __set__(self, obj, value):
        raise AttributeError, "Can't change it"

    def __delete__(self):
        raise AttributeError, "Can't delete as well"

class SecurityDescriptor:
    """ SecurityDescriptor is used to return the
    checker for the proxied object when its an instance,
    and return the checker for the class, when its a class
    """

    def __init__(self, proxied_attr='__proxied__'):
        self.proxied_attr = proxied_attr

    def __get__(self, inst, cls=None):
        if inst is None:
            return selectChecker(cls)
        else:
            # If we are using a SQL Schema, then the field
            # will be wrapped on a ContainerProxy, and
            # then selectChecker will get the wrong checker.
            obj = containerProxiedObject(getattr(inst, self.proxied_attr))
            return selectChecker(obj)

class ClassDescriptor:
    """ ClassDescriptor is used to return the
    real class when a class is being used, and return
    the proxied object's class when its an instance.
    """

    def __init__(self, proxied_attr='__proxied__'):
        self.proxied_attr = proxied_attr

    def __get__(self, inst, cls=None):
        if inst is None:
            return cls
        else:
            return getattr(inst, self.proxied_attr).__class__
