##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: subscriber.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

from zope.app.tests import setup
from zope.app import zapi
from zope.app.servicenames import Utilities, \
     Authentication
from zope.app.utility import LocalUtilityService
from zope.app.pluggableauth import PluggableAuthenticationService

from sip.interfaces import IAppRoot
from sip.interfaces.utilities import ISplitDerive, IQuery
from sip.utilities.splitderive import SplitDerive
from sip.utilities.query import Query
from sip import SecurityPrincipal, SecurityRole

from sqlos.auth import SQLObjectPrincipalSource
from sqlos.interfaces import ISQLObject

def appRootCreationSubscriber(event):
    obj = event.object
    if not IAppRoot.providedBy(obj): return

    # Create a service manager at the approot level
    mgr = setup.createServiceManager(obj)

    # Create a pluggable auth service at the root folder
    setup.addService(mgr, Authentication,
                     PluggableAuthenticationService())

    # Then add a SQLObject PrincipalSource
    auth = zapi.getService(Authentication, context=obj)
    prin_source = SQLObjectPrincipalSource()
    auth.addPrincipalSource('sqlob', prin_source)

    # Setup Utility and Interface Services
    setup.addService(mgr, Utilities, LocalUtilityService())

    # Setup our own utilities
    setup.addUtility(mgr, '', ISplitDerive, SplitDerive(), 'split-derive')
    setup.addUtility(mgr, '', IQuery, Query(), 'query')

def sqlobjectModifiedSubscriber(event):
    obj = event.object
    if not ISQLObject.providedBy(obj): return

    if isinstance(obj, (SecurityPrincipal, SecurityRole)):
        from sip.security.source import securityCache
        securityCache.clear()
        from sip.security.principalrole import securityCache
        securityCache.clear()
