##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: roles.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

import unittest

from zope.app import zapi
from zope.app.servicenames import Authentication
from zope.exceptions import ForbiddenAttribute, Unauthorized
from zope.security.proxy import ProxyFactory
from sip.browser import api
from sip.ftests.base import Base

class Roles(Base):

    full_setup = True
    level = 2

    def _roleListProjects(self):
        app = zapi.traverse(self.rootFolder, 'sip')
        view = zapi.getView(app, 'index.html', self.request)
        view = ProxyFactory(view)
        return view()

    def _roleSeeProjects(self):
        projects = zapi.traverse(self.rootFolder, 'sip/projects')
        projects = ProxyFactory(projects)
        return [v for v in projects.values()]

    def _roleListLocations(self):
        url = 'sip/projects/sip.Project.%s/locations' % self.ids['project']
        locations = zapi.traverse(self.rootFolder, url)
        view_name = zapi.getDefaultViewName(locations, self.request)
        view = zapi.getView(locations, view_name, self.request)
        view = ProxyFactory(view)
        return view()

    def _roleEnterLocation(self):
        url = 'sip/projects/sip.Project.%s/locations/sip.Location.%s' % (
            self.ids['project'], self.ids['location'])
        location = zapi.traverse(self.rootFolder, url)
        view_name = zapi.getDefaultViewName(location, self.request)
        view = zapi.getView(location, view_name, self.request)
        view = ProxyFactory(view)
        return view()

    def _roleListSamples(self):
        project = zapi.traverse(self.rootFolder, 'sip/projects/sip.Project.%s' %
                            self.ids['project'])
        view = zapi.getView(project, 'sample_listing.html', self.request)
        view = ProxyFactory(view)
        return view()

    def _roleAddSample(self):
        url = 'sip/projects/sip.Project.%s/samples' % self.ids['project']
        samples = zapi.traverse(self.rootFolder, url)
        adding = zapi.getView(samples, '+', self.request)
        adding = api.contained(adding, samples, '+')
        add = zapi.getView(adding, 'add_sample.html', self.request)
        add = ProxyFactory(add)
        return add

    def _roleEditSample(self):
        url = 'sip/projects/sip.Project.%s/samples/sip.Sample.%s' % (
            self.ids['project'], self.ids['sample'])
        self.request._traversed_names = url.split('/')
        sample = zapi.traverse(self.rootFolder, url)
        edit = zapi.getView(sample, 'edit_sample.html', self.request)
        edit = ProxyFactory(edit)
        return edit

    def _roleAdminProject(self):
        url = 'sip/projects/sip.Project.%s' % self.ids['project']
        self.request._traversed_names = url.split('/')
        project = zapi.traverse(self.rootFolder, url)
        view = zapi.getView(project, 'admin_project.html', self.request)
        view = ProxyFactory(view)
        return view

    def _roleAppSetup(self):
        url = 'sip'
        app = zapi.traverse(self.rootFolder, url)
        view = zapi.getView(app, 'app_setup.html', self.request)
        view = ProxyFactory(view)
        return view

    def _roleEditProject(self):
        url = 'sip/projects/sip.Project.%s' % self.ids['project']
        project = zapi.traverse(self.rootFolder, url)
        view = zapi.getView(project, 'edit.html', self.request)
        view = ProxyFactory(view)
        return view

    def _roleViewProjectsContents(self):
        url = 'sip/projects'
        projects = zapi.traverse(self.rootFolder, url)
        view = zapi.getView(projects, 'contents.html', self.request)
        view = ProxyFactory(view)
        return view
