##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: test_appmanagerrole.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

import unittest

from zope.app import zapi
from zope.app.servicenames import Authentication
from zope.exceptions import ForbiddenAttribute, Unauthorized
from zope.app.securitypolicy.interfaces import IPrincipalRoleManager
from sip.ftests.test_projectmanagerrole import TestPMRole
from sip.ftests.test_reportrole import TestSQLRole

class TestAMRole(TestPMRole):

    _roles = ['sip.Report', 'sip.LabTechnician',
              'sip.ProjectManager', 'sip.AppManager']
    _principal = 'sip.app_manager'

    def testAppSetup(self):
        view = self._roleAppSetup()
        self.failUnless(view())

class TestSQLAMRole(TestSQLRole, TestAMRole):

    def _setupTestUser(self, existing=False, use_root=True):
        super(TestSQLAMRole, self)._setupTestUser(existing, use_root)

    def testSeeProjects(self):
        projects = self._roleSeeProjects()
        self.failUnless(len(projects) == 2)
        # Some role juggling to juggle the roles of the
        # test user.
        self._setupSystemUser()
        prm = IPrincipalRoleManager(self.rootFolder['sip'])
        prm.unsetRoleForPrincipal('sip.Report',
                                  self._principal)
        self._setupTestUser(existing=True)
        projects = self._roleSeeProjects()
        self.failUnless(len(projects) == 0)
        self._setupSystemUser()
        prm = IPrincipalRoleManager(self.project)
        prm.assignRoleToPrincipal('sip.Report', self._principal)
        self._setupTestUser(existing=True)
        projects = self._roleSeeProjects()
        # Should see only one project now, the one where she
        # has the Report role.
        self.failUnless(len(projects) == 1)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestAMRole))
    suite.addTest(unittest.makeSuite(TestSQLAMRole))
    return suite

if __name__ == '__main__':
    unittest.main()
