##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: test_location.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

import unittest

from zope.app import zapi
from zope.app.container.interfaces import IReadContainer
from sip.ftests.base import Base

class TestLocation(Base):
    level = 2
    use_test_user = False

    def testAddLocation(self):
        containertypes = zapi.traverse(self.rootFolder, 'sip/containertypes')
        self.assertEquals(len(containertypes.keys()), 0)
        ct = self._addContainerType()
        self.assertEquals(len(containertypes.keys()), 1)

        locations = zapi.traverse(self.rootFolder, 'sip/locations')
        self.assertEquals(len(locations.keys()), 0)
        self._addLocation(type_id=ct.id)
        self.assertEquals(len(locations.keys()), 1)

    def testAddContainer(self):
        ct = self._addContainerType()
        location = self._addLocation(type_id=ct.id)
        adapted = IReadContainer(location)
        self.assertEquals(len(adapted),0)
        container = self._addContainer(location, ct.id)
        self.assertEquals(len(adapted),1)

    def testAssignLocation(self):
        ct = self._addContainerType()
        location = self._addLocation(type_id=ct.id)
        project = self._addProject()
        valid_locations = project.valid_locations
        locations = zapi.traverse(project, 'locations')
        self.assertEquals(len(valid_locations), 0)
        self.assertEquals(len(locations), 0)
        project.valid_locations = [location.id]
        valid_locations = project.valid_locations
        self.assertEquals(len(valid_locations), 1)
        self.assertEquals(len(locations), 1)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestLocation))
    return suite

if __name__ == '__main__':
    unittest.main()
