##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: test_projectmanagerrole.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

import unittest

from zope.app import zapi
from zope.app.servicenames import Authentication
from zope.exceptions import ForbiddenAttribute, Unauthorized
from sip.ftests.test_labtechnicianrole import TestLabTechRole
from sip.ftests.test_reportrole import TestSQLRole
from sip.interfaces.adapter import IPrincipalRolesHelper

class TestPMRole(TestLabTechRole):

    _roles = ['sip.Report', 'sip.LabTechnician', 'sip.ProjectManager']
    _principal = 'sip.project_manager'

    def testAdminProject(self):
        view = self._roleAdminProject()
        self.failUnless(view())

    def testEditProject(self):
        view = self._roleEditProject()
        self.failUnless(view())

class TestSQLPMRole(TestSQLRole, TestPMRole):

    def testAvailableRoles(self):
        project_id = "sip.Project.%s" % self.ids['project']
        project = self.rootFolder['sip']['projects'][project_id]
        adapter = IPrincipalRolesHelper(project)
        roles = adapter.availableRoles()
        self.failIf('sip.AppManager' in roles)
        self.failUnless('sip.ProjectManager' in roles)
        self.failUnless('sip.LabTechnician' in roles)
        self.failUnless('sip.Report' in roles)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestPMRole))
    suite.addTest(unittest.makeSuite(TestSQLPMRole))
    return suite

if __name__ == '__main__':
    unittest.main()
