##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: test_reportrole.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

import unittest

from zope.app import zapi
from zope.security.management import newInteraction, endInteraction
from zope.app.servicenames import Authentication
from zope.exceptions import ForbiddenAttribute, Unauthorized
from zope.app.securitypolicy.interfaces import IPrincipalRoleManager
from zope.app.securitypolicy.principalrole import principalRoleManager
from zope.proxy import removeAllProxies

from sip.ftests.roles import Roles
from sip.ftests.base import ParticipationStub
from sip.browser import api

class TestReportRole(Roles):

    _roles = ['sip.Report']
    _principal = 'sip.report'

    def _setupTestUser(self):
        auth_service = zapi.getService(Authentication)
        user = auth_service.getPrincipal(self._principal)
        endInteraction()
        newInteraction(ParticipationStub(user))

    def testListProjects(self):
        self.failUnless(self._roleListProjects())

    def testSeeProjects(self):
        projects = self._roleSeeProjects()
        self.failUnless(len(projects) == 2)
        project = projects[0]
        principalRoleManager.unsetRoleForPrincipal('sip.Report',
                                                   self._principal)
        projects = self._roleSeeProjects()
        self.failUnless(len(projects) == 0)
        self._setupSystemUser()
        prm = IPrincipalRoleManager(project)
        prm.assignRoleToPrincipal('sip.Report', self._principal)
        self._setupTestUser()
        projects = self._roleSeeProjects()
        # Should see only one project now, the one where she
        # has the Report role.
        self.failUnless(len(projects) == 1)

    def testListLocations(self):
        self.failUnless(self._roleListLocations())

    def testEnterLocation(self):
        self.failUnless(self._roleListLocations())

    def testListSamples(self):
        self.failUnless(self._roleListSamples())

    def testAddSample(self):
        self.assertRaises(Unauthorized, self._roleAddSample())

    def testEditSample(self):
        self.assertRaises(Unauthorized, self._roleEditSample())

    def testAdminProject(self):
        self.assertRaises(Unauthorized, self._roleAdminProject())

    def testAppSetup(self):
        self.assertRaises(Unauthorized, self._roleAppSetup())

    def testEditProject(self):
        self.assertRaises(Unauthorized, self._roleEditProject())

    def testViewProjectsContents(self):
        self.failUnless(self._roleViewProjectsContents()())

class SQLRole:
    """ Mixin test for testing roles and users in SQL. One of the
    mixed-in classes must define _principal and _roles, which are the
    principal to be tested and the roles to give to the principal on
    the project.
    """

    _principal_id = None

    def _setupTestUser(self, existing=False, use_root=False):
        if not self._principal_id:
            self._principal_id = "sql-%s" % self._principal
        if not existing:
            self._addPrincipal(username=self._principal_id)
        root = self.rootFolder['sip']
        auth_service = zapi.getService(Authentication, context=root)
        user = list(auth_service.getPrincipals(self._principal_id))[0]
        prin_id = self._principal = user.id
        if not existing:
            prm = IPrincipalRoleManager(root)
            prm.assignRoleToPrincipal('sip.User', prin_id)
            target = root
            if not use_root:
                target = self.project
            prm = IPrincipalRoleManager(target)
            for role in self._roles:
                prm.assignRoleToPrincipal(role, prin_id)
        endInteraction()
        newInteraction(ParticipationStub(user))


class TestSQLRole(SQLRole):

    def testSeeProjects(self):
        self._setupTestUser(existing=True, use_root=False)
        projects = self._roleSeeProjects()
        self.failUnless(len(projects) == 1)
        # Some role juggling to juggle the roles of the
        # test user.
        self._setupSystemUser()
        prm = IPrincipalRoleManager(self.project)
        prm.unsetRoleForPrincipal('sip.Report', self._principal)
        self._setupTestUser(existing=True, use_root=False)
        projects = self._roleSeeProjects()
        self.failUnless(len(projects) == 0)
        self._setupSystemUser()
        prm = IPrincipalRoleManager(self.project)
        prm.assignRoleToPrincipal('sip.Report', self._principal)
        self._setupTestUser(existing=True, use_root=False)
        projects = self._roleSeeProjects()
        # Should see only one project now, the one where she
        # has the Report role.
        self.failUnless(len(projects) == 1)


class TestSQLReportRole(TestSQLRole, TestReportRole): pass

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestReportRole))
    suite.addTest(unittest.makeSuite(TestSQLReportRole))
    return suite

if __name__ == '__main__':
    unittest.main()
