##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: test_root.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

import unittest

from zope.app import zapi
from zope.app.container.interfaces import IReadContainer
from zope.proxy import sameProxiedObjects

from sip.ftests.base import Base

virtual = ['containertypes',
           'concentrationunits',
           'weightunits',
           'technicians',
           'volumeunits',
           'locations',
           'projects',
           'originsamplealiquots',
           'sampletypes',
           'principals',
           'subjecttypes'
           ]

project_virtual = ['aliquots',
                   'locationsproject',
                   'samples',
                   'schemas',
                   ]

class AppRootSetup(Base):
    level = 2

    def testRoot(self):
        sip = zapi.traverse(self.rootFolder, 'sip')
        for v in virtual:
            self.failUnless(v in sip.keys())

    def testProjects(self):
        project = self._addProject()
        project = IReadContainer(project)
        for v in project_virtual:
            self.failUnless(v in project.keys())

    def testConcentrationUnits(self):
        cunits = zapi.traverse(self.rootFolder, 'sip/concentrationunits')
        self.assertEquals(len(cunits),24)
        unit = self._addConcentrationUnit()
        labels = [x.label for x in cunits.values()]
        self.failUnless(unit.label in labels)

    def testVolumeUnits(self):
        vunits = zapi.traverse(self.rootFolder, 'sip/volumeunits')
        self.assertEquals(len(vunits),7)
        unit = self._addVolumeUnit()
        labels = [x.label for x in vunits.values()]
        self.failUnless(unit.label in labels)

    def testWeightUnits(self):
        wunits = zapi.traverse(self.rootFolder, 'sip/weightunits')
        self.assertEquals(len(wunits),7)
        unit = self._addWeightUnit()
        labels = [x.label for x in wunits.values()]
        self.failUnless(unit.label in labels)

    def testContext(self):
        projects1 = zapi.traverse(self.rootFolder, 'sip/projects')
        project = self._addProject()
        u1 =  zapi.getView(projects1, 'absolute_url', self.request)()
        projects2 = zapi.getParent(project)
        u2 = zapi.getView(projects2, 'absolute_url', self.request)()
        self.assertEquals(u1, u2)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(AppRootSetup))
    return suite

if __name__ == '__main__':
    unittest.main()
