##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: test_rootcreation.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

from unittest import TestSuite, main, makeSuite
from zope.interface import Interface, implements

from zope.component.service import serviceManager

from zope.app import zapi
from zope.app.container.interfaces import IObjectAddedEvent

from zope.app.servicenames import Utilities, Authentication

from sip.root import AppRoot
from sip.ftests.base import Base
from sip.interfaces.utilities import ISplitDerive, IQuery
from sip.browser.api import queryService

def gp(ob):
    _gp = zapi.getParent
    return _gp(_gp(ob))

class AppRootCreation(Base):
    level = 1

    def test_event_services_creation(self):
        # now for real!
        app = self.sip

        sm = zapi.getServices(app)
        # Check that we didn't got the global SM by mistake
        self.failIf(sm is serviceManager)

        auth = queryService(Authentication, context=app)
        self.failIf(auth is None)
        self.failUnless(gp(auth) is sm)
        self.failUnless('sqlob' in auth)

        util = queryService(Utilities, context=app)
        self.failIf(util is None)
        self.failUnless(gp(util) is sm)

        sq = zapi.queryUtility(IQuery, context=app)
        self.failIf(sq is None)
        self.failUnless(gp(sq) is sm)

        split = zapi.queryUtility(ISplitDerive, context=app)
        self.failIf(split is None)
        self.failUnless(gp(split) is sm)


def test_suite():
    return TestSuite((
        makeSuite(AppRootCreation),
        ))

if __name__=='__main__':
    main(defaultTest='test_suite')
