##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: test_sqlschema.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

import unittest

from transaction import get_transaction
from zope.schema import getFieldsInOrder, getFieldNamesInOrder
from zope.app import zapi

from sip.schema.tests.test_schemautility import SchemaUtilityTests
from sip.schema.interfaces import IMutableSchema

from sip.ftests.base import Base
from sip.browser import api

class SQLSchemaUtilityTest(SchemaUtilityTests, Base):
    level = 2
    use_test_user = False

    def setUp(self):
        Base.setUp(self)
        try:
            SchemaUtilityTests._additionalSetup(self)
        except:
            get_transaction().abort()
            raise

    def _createSchemaUtility(self):
        pf = api.factory('sip.Project', context=self.sip)
        p = pf(name='SchemaTestProject', commit_message='Test')
        stf = api.factory('sip.SampleType', context=self.sip)
        st = stf(name='Blood', commit_message='Test')
        factory = api.factory('sip.SampleTypeSchema', context=self.sip)
        schema = factory(sample_type_id=st.id,
                             project_id=p.id)

        sts = api.contained(schema, self.sip, 'schema')
        return IMutableSchema(sts)

    def test_traverseToField(self):
        s = self.s
        s.addField(u'alpha', self.alpha)
        f1 = s[u'alpha']
        order = f1.order
        f1 = zapi.traverse(s, 'alpha')
        self.assertEquals(f1.order, self.alpha.order)
        title = zapi.traverse(f1, 'title')
        self.assertEquals(title, self.alpha.title)
        fields = getFieldsInOrder(s)
        for k, v in fields:
            self.failUnless(v.title is not None)

    def test_schemaFieldItems(self):
        s = self.s
        s.addField(u'alpha', self.alpha)
        fields = getFieldNamesInOrder(s)
        for k, v in s._attrs.items():
            self.failUnless(k in fields, (k, fields))

    def test_default_not_required(self):
        from zope.schema.interfaces import IText, ITextLine, IBool, \
             IInt, IFloat, IDate, IDatetime
        for iface in (IText, ITextLine, IBool, IInt, IFloat,
                      IDate, IDatetime):
            self.failUnless(iface['default'].required == False)

    def tearDown(self):
        Base.tearDown(self)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SQLSchemaUtilityTest))
    return suite

if __name__ == '__main__':
    unittest.main()
