##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: test_userrole.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

import unittest

from zope.app import zapi
from zope.security.management import newInteraction, endInteraction
from zope.app.servicenames import Authentication
from zope.exceptions import ForbiddenAttribute, Unauthorized
from zope.app.securitypolicy.interfaces import IPrincipalRoleManager
from sip.ftests.roles import Roles
from sip.ftests.base import ParticipationStub
from sip.browser import api

class TestUserRole(Roles):

    _role = 'sip.Report'
    _principal = 'sip.user'

    def _setupTestUser(self):
        auth_service = zapi.getService(Authentication)
        user = auth_service.getPrincipal(self._principal)
        endInteraction()
        newInteraction(ParticipationStub(user))

    def testSeeProjects(self):
        projects = self._roleSeeProjects()
        self.failUnless(len(projects) == 0)
        self._setupSystemUser()
        projects = self._roleSeeProjects()
        project = projects[0]
        prm = IPrincipalRoleManager(project)
        prm.assignRoleToPrincipal('sip.Report', self._principal)
        self._setupTestUser()
        projects = self._roleSeeProjects()
        # Should see only one project now, the one where she
        # has the Report role.
        self.failUnless(len(projects) == 1)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestUserRole))
    return suite

if __name__ == '__main__':
    unittest.main()
