##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: add.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

from zope.interface import Interface
from zope.schema import TextLine, Bool, Text, Field

from sip.i18n import _
from sip.fields import FieldTypeField

class IFieldAdd(Interface):

    id = TextLine(
        title=_('Name'),
        default=u"",
        required=True,
        )

    type = FieldTypeField(
        title=_('Field Type'),
        required=True,
        allowed_names=['Text Field',
                       'Integer Field',
                       'Boolean Field',
                       'Date Field',
                       'Text Line Field',
                       'Float Field',
                       ],
        vocabulary='sip.vocab.field_types'
        )

    title = TextLine(
        title=_('Title'),
        description=_('A short summary or label'),
        default=u"",
        required=True,
        )

    description = Text(
        title=_('Description'),
        description=_('A description of the field'),
        default=u"",
        required=False,
        )

    required = Bool(
        title=_('Required'),
        description=_('Whether a field requires its value to exist.'),
        default=True,
        required=False)

    usetales = Bool(
        title=_('Use TALES Validation?'),
        description=_('Should this field allow TALES Validation?'),
        required=False,
        default=False
        )

    default = TextLine(
        title=_('Default Value'),
        description=_('Default value for this field, if none is provided'),
        required=False,
        default=u''
        )
