##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: fields.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

from zope.interface import implements, Interface
from zope.schema import TextLine, Bool, Text, Field
from zope.schema.interfaces import IText
from zope.tales.tales import Undefined, CompilerError
from zope.app.pagetemplate.engine import Engine

from sip.i18n import _

class ICommitMessage(IText):
    """ A Commit Message field. Nothing special, just a
    marker interface to allow using a special widget. """

class IBRField(Interface):
    """marker interface for linebreaks in forms"""

class ISubjectTypeField(Interface):
    """marker to use radio widget"""

class IROSubjectTypeWidget(Interface):
    """marker interface"""

def TALESExpressionConstraint(value):
    assert not value.startswith('string'), ('We only accept python: '
                                            'TALES expressions')
    if not value.startswith('python:'):
        value = "python: %s" % value
    try:
        compiled = Engine.compile(value)
    except (SyntaxError, CompilerError, Undefined):
        return False
    else:
        return True

class ITALESExpressionField(Interface):
    """ marker interface """

class IInAndOutField(Interface):
    """marker interface"""

class TALESExpressionField(TextLine):
    implements(ITALESExpressionField)

class ITALESConstrainedField(Interface):
    """A marker interface for fields which have a TALES
    constraint"""
    expression = TALESExpressionField(title=_('TALES Expression'),
                          description=_(('A TALES Expression. '
                                         'the variables you have available '
                                         'are: value, nothing.')),
                          constraint=TALESExpressionConstraint,
                          required=False)

    def __call__(value):
        """Evaluate the TALES expression, using value
        and return True if its a valid value, or raise a
        ValidationError."""
