##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: location.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

from zope.interface import Interface
from zope.schema import TextLine, Int, Bool
from sip.fields import ContainerTypeField
from sip.interfaces import ISIPSchema, Unique, FieldFactory
from sip.descriptors import BoundCallableDescriptor
from sip.i18n import _

class ILocation(Interface):
    """A marker interface for locations"""

    def getRootContainer():
        """ Returns the first container in this location which has
        no parent (thus, being a root container)"""

    def getRootId():
        """ returns the container_id of the root container"""

class ILocationSchema(ISIPSchema):
    """ """
    name = FieldFactory(
        TextLine,
        title=_('Location Name'),
        description=_('A string to serve as identifier for this location'),
        required=True,
        max_length=50,
        constraint=BoundCallableDescriptor(
        Unique(className='sip.Location', columnName='name'))
        )

    type_id = ContainerTypeField(
        title=_('Container Type'),
        description=_('The container type for this location'),
        vocabulary='sip.vocab.container_types')

    building = TextLine(title=_('Building'),
                        description=_('Building where this location pertains'),
                        required=True,
                        max_length=255,
                        )
    room = TextLine(title=_('Room'),
        description=_('Room where this location is located inside the building'),
                        required=True,
                        max_length=255,
                        )
    temperature = Int(title=_('Temperature'),
                           description=_('The temperature for this location'),
                      required=False)
