##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: principal.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

from zope.interface import Interface
from zope.schema import TextLine, Password
from sip.i18n import _
from sqlos.interfaces.auth import IPrincipal as IPrincipalBase
from sip.interfaces import ISQLSchema, FieldFactory
from sip.descriptors import BoundCallableDescriptor
from sip.utils import Unique, PasswordMatch

class IPrincipalInfo(ISQLSchema):
    """ A Principal! """

    fullname = TextLine(title=_('Full Name'),
                        required=True)

    username = FieldFactory(
        TextLine,
        title=_('Username'),
        description=_('The user login name'),
        max_length=50,
        required=True,
        constraint=BoundCallableDescriptor(
        Unique(className='sip.Principal', columnName='username'))
        )

class IPrincipalPassword(Interface):
    """ Split into a different interface to ease protecting
    the password by a different permission """

    password = Password(title=_('Password'),
                        required=True)

    confirm_pass = FieldFactory(
        Password,
        title=_('Confirm Password'),
        required=True,
        constraint=BoundCallableDescriptor(
        PasswordMatch())
        )

class IPrincipal(IPrincipalInfo, IPrincipalPassword, IPrincipalBase):
    """ Principal Schema """
