##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: sample.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

from zope.interface import Interface, Attribute
from zope.schema import TextLine, Text, Datetime, Bool
from zope.schema import Choice
from sip.fields import SubjectTypeField
from sip.i18n import _
from sip.interfaces import ISIPSchema, Unique, FieldFactory
from sip.descriptors import BoundCallableDescriptor

from calendar_widget import CalendarDate

class ISampleSchema(ISIPSchema):
    """ """

    name = FieldFactory(
        TextLine,
        title=_('Sample Name'),
        description=_("The name of the sample"),
        max_length=255,
        required=False,
        constraint=BoundCallableDescriptor(
        Unique(className='sip.Sample', columnName='name', byProject=True))
        )

    other_id = TextLine(
        title=_('Other ID'),
        description=_("A outside parties identification for this sample"),
        max_length=255,
        required=False)

    type_id = Choice(
        title=_('Sample Type'),
        description=_('The type (kind) of this sample'),
        required=True,
        vocabulary=u'sip.vocab.project_sample_types')

    technician_id = Choice(
        title=_('Technician ID'),
        description=_('The technician who drew this sample.'),
        readonly=False,
        required=False,
        vocabulary='sip.vocab.project_technicians')

    subject_type_id = SubjectTypeField(
        title=_('Subject Type'),
        description=_('Type of subject'),
        readonly=False,
        required=True,
        default=1,
        vocabulary='sip.vocab.subject_types')

    subject_id = TextLine(
        title=_('Subject ID'),
        description=_("Unique identifier for this subject"),
        required=False)

    collection_date = CalendarDate(
        title=_('Date Collected'),
        description=_('Date this sample was collected'),
        required=False)

    comments = Text(
        title=_('Comments'),
        description=_("Ad hoc comments about this sample."),
        required=False)

class ISampleDate(Interface):
    """ """

    date_generated = Datetime(
        title=_('Date Generated'),
        description=_('Date this sample was generated'),
        required=False)

    derived = Attribute('Sample is Derived')


class ISample(ISampleSchema, ISampleDate):
    """ """

    derived = Attribute('Sample is Derived')

class ISampleAdd(Interface):
    """ Marker interface for adding a sample """
