##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: schema.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

from zope.interface import Interface, Attribute
from zope.schema import TextLine, Text, Datetime, Int
from zope.schema import Bool

from sip.i18n import _
from sip.interfaces import ISQLSchema

class ISchemaField(ISQLSchema):

    title = TextLine(
        title=_('Title'),
        description=_('A short summary or label'),
        default=u"",
        required=False,
        )

    description = Text(
        title=_('Description'),
        description=_('A description of the field'),
        default=u"",
        required=False,
        )

    required = Bool(
        title=_('Required'),
        description=_('Whether a field requires its value to exist.'),
        default=True)

    readonly = Bool(
        title=_('Read Only'),
        description=_('Whether a the field value can be changed or not.'),
        required=False,
        default=False)

    order = Int(
        title=_('Field Order'),
        description=_(('The order attribute can be used to '
                       'determine the order in which fields in '
                       'a schema were defined. If one field is '
                       'created after another (in the same thread), '
                       'its order will be greater. (Fields in '
                       'separate threads could have the same order.)')),
        required=True,
        readonly=True,
        )
