##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
from zope.interface import Interface, Attribute
from zope.app.container.constraints import ItemTypePrecondition

from sip.interfaces.project import IProject
from sip.interfaces.aliquot import IAliquot
from sip.interfaces.sample import ISample
from sip.interfaces.principal import IPrincipal
from sip.interfaces.location import ILocation
from sip.interfaces.container import IContainer
from sip.interfaces.technician import ITechnician
from sip.interfaces.sampletypeschema import ISampleTypeSchema
from sip.interfaces.origin_sample_aliquots import IOriginSampleAliquots

from sip.interfaces.units import IVolumeUnit
from sip.interfaces.units import IWeightUnit
from sip.interfaces.units import IConcentrationUnit

from sip.interfaces.sampletype import ISampleType
from sip.interfaces.subjecttype import ISubjectType
from sip.interfaces.containertype import IContainerType

from sip.interfaces.location_project import ILocationProject
from sip.interfaces.technician_project import ITechnicianProject
from sip.interfaces.sampletype_project import ISampleTypeProject
from sip.interfaces.shippinginfo import ICarrier

from sqlos.interfaces.container import ISQLObjectContainer

class ISource(ISQLObjectContainer):
    """Marker interface"""

    use_project = Attribute("Try to find a project and use it's "
                            "id to constrain queries?")
    constraint = Attribute("A constraint to restrict the values "
                           "returned by the container")

class IProjectSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(IProject)

class IAliquotSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(IAliquot)

class ISampleSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(ISample)

class IPrincipalSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(IPrincipal)

class ILocationSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(ILocation)

class IContainerSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(IContainer)

class IContainerAliquotSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(IContainer, IAliquot)

class ITechnicianSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(ITechnician)

class ISampleTypeSchemaSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(ISampleTypeSchema)

class IOriginSampleAliquotsSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(IOriginSampleAliquots)

class IVolumeUnitSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(IVolumeUnit)

class IWeightUnitSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(IWeightUnit)

class IConcentrationUnitSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(IConcentrationUnit)

class ISampleTypeSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(ISampleType)

class ISubjectTypeSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(ISubjectType)

class IContainerTypeSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(IContainerType)

class ILocationProjectSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(ILocationProject)

class ITechnicianProjectSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(ITechnicianProject)

class ICarrierSource(ISource):

    def __setitem__(name, obj):
        """Add a new object"""

    __setitem__.precondition = ItemTypePrecondition(ICarrier)
