##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: units.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

from zope.schema import Bool, Choice
from zope.schema import TextLine, Text, Datetime, Int, Float
from zope.interface import Interface, Attribute

from sip.i18n import _
from sip.interfaces import ISIPSchema, Unique, FieldFactory
from sip.descriptors import BoundCallableDescriptor


class IUnit(ISIPSchema):

    name = Attribute('Name')

    exponent = Float(title=_('Exponent'),
                   required=True)

class IConcentrationUnit(IUnit):
    """ """

    label = FieldFactory(
        TextLine,
        title=_('Label'),
        required=True,
        constraint=BoundCallableDescriptor(
        Unique(className='sip.ConcentrationUnit', columnName='label'))
        )

class IWeightUnit(IUnit):
    """ """

    label = FieldFactory(
        TextLine,
        title=_('Label'),
        required=True,
        constraint=BoundCallableDescriptor(
        Unique(className='sip.WeightUnit', columnName='label'))
        )

class IVolumeUnit(IUnit):
    """ """
    label = FieldFactory(
        TextLine,
        title=_('Label'),
        required=True,
        constraint=BoundCallableDescriptor(
        Unique(className='sip.VolumeUnit', columnName='label'))
        )

class IAppConcentrationUnits(Interface):
    """Available locations on the App"""

    available_concentrationunits = Choice(
        title=_('Available Concentration Units'),
        description=_(('Concentration Units that '
                       'are available for use')),
        required=True,
        vocabulary='sip.vocab.available_concentration_units')

class IAppWeightUnits(Interface):
    """Available locations on the App"""

    available_weightunits = Choice(
        title=_('Available Weight Units'),
        description=_('Weight Units that are available for use'),
        required=True,
        vocabulary='sip.vocab.available_weight_units')

class IAppVolumeUnits(Interface):
    """Available locations on the App"""

    available_volumeunits = Choice(
        title=_('Available Volume Units'),
        description=_('Volume Units that are available for use'),
        required=True,
        vocabulary='sip.vocab.available_volume_units')
