##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: root.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""

__metaclass__ = type

from persistent import Persistent

from zope.interface import implements
from zope.app import zapi
from zope.app.container.contained import Contained
from zope.app.site.servicecontainer import ServiceManagerContainer
from zope.proxy import removeAllProxies

from sip.interfaces import IAppRoot, IAppProjects, \
     IAppLocations, IAppContainerTypes, IAppPrincipals
from sip.interfaces.units import IAppConcentrationUnits, \
     IAppWeightUnits, IAppVolumeUnits
from sip.browser import api

from sqlos.interfaces import ISQLAttributeAnnotatable

class AppRoot(Persistent, ServiceManagerContainer, Contained):
    """ """

    implements(IAppRoot, IAppProjects, IAppLocations,
               IAppConcentrationUnits,
               IAppWeightUnits, IAppVolumeUnits,
               IAppContainerTypes,
               IAppPrincipals,
               ISQLAttributeAnnotatable)

    def keys(self):
        return [k for k, v in self.items()]

    def __iter__(self):
        return iter(self.keys())

    def values(self):
        for k, v in self.items():
            yield v

    def items(self):
        cr = zapi.getService('AppContainerRegistry')
        items = cr.queryContainers(self).iteritems()
        return api.containedItems(items, self)

    def __getitem__(self, name):
        v = self.get(name, None)
        if v is None:
            raise KeyError, name
        return v

    def get(self, name, default=None):
        cr = zapi.getService('AppContainerRegistry')
        item = cr.queryContainers(self).get(name, default)
        if item is not default:
            return api.contained(item, self, name=name)
        return item

    def __contains__(self, name):
        return self.get(name, None) is not None

    def __len__(self):
        return len(self.keys())

    def get_projects(self):
        projects = zapi.traverse(self, 'projects')
        return projects.keys()

    def set_projects(self, value):
        raise AttributeError

    available_projects = property(get_projects, set_projects)

    def get_locations(self):
        locations = zapi.traverse(self, 'locations')
        return locations.keys()

    def set_locations(self, value):
        raise AttributeError

    available_locations = property(get_locations, set_locations)

    def get_containertypes(self):
        containertypes = zapi.traverse(self, 'containertypes')
        return containertypes.keys()

    def set_containertypes(self, value):
        raise AttributeError

    available_containertypes = property(get_containertypes, set_containertypes)

    def get_concentrationunits(self):
        source = zapi.traverse(self, 'concentrationunits')
        return source.keys()

    def set_concentrationunits(self, value):
        raise AttributeError

    available_concentrationunits = property(get_concentrationunits,
                                            set_concentrationunits)

    def get_weightunits(self):
        source = zapi.traverse(self, 'weightunits')
        return source.keys()

    def set_weightunits(self, value):
        raise AttributeError

    available_weightunits = property(get_weightunits, set_weightunits)

    def get_volumeunits(self):
        source = zapi.traverse(self, 'volumeunits')
        return source.keys()

    def set_volumeunits(self, value):
        raise AttributeError

    available_volumeunits = property(get_volumeunits, set_volumeunits)

    def get_principals(self):
        source = zapi.traverse(self, 'principals')
        return [p.name for p in source.values()]

    def set_principals(self, value):
        raise AttributeError

    available_principals = property(get_principals, set_principals)
