##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: traversal.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""
from zope.interface import implements
from zope.component import getDefaultViewName, queryView
from zope.publisher.interfaces.browser import IBrowserPublisher

from zope.exceptions import NotFoundError

from zope.app.traversing.interfaces import ITraversable
from zope.app.traversing.namespace import UnexpectedParameters
from zope.app.location.interfaces import ILocation

from sip.schema.interfaces import IMutableSchema

_marker = object()

class SchemaFieldTraverser:
    implements(IBrowserPublisher)
    __used_for__ = IMutableSchema

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def publishTraverse(self, request, name):
        subob = self.context.get(name, None)

        if subob is None:

            view = queryView(self.context, name, request)
            if view is not None:
                if ILocation.providedBy(view):
                    view.__parent__ = self.context
                    view.__name__ = name

                return view

            raise NotFoundError(self.context, name, request)

        return subob

    def browserDefault(self, request):
        c = self.context
        view_name = getDefaultViewName(c, request)
        view_uri = "@@%s" % view_name
        return c, (view_uri,)

class SchemaFieldTraversable:
    """Traverses Schema Fields.
    """

    implements(ITraversable)
    __used_for__ = IMutableSchema

    def __init__(self, context):
        self._context = context

    def traverse(self, name, furtherPath):
        subobj = self._context.get(name, _marker)
        if subobj is _marker:
            raise NotFoundError, name

        return subobj
