##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: principalrole.py,v 1.2 2005/10/21 00:11:35 dreamcatcher Exp $
"""

__metaclass__ = type

from zope.component import getAdapter
from zope.interface import implements

from zope.app.annotation.interfaces import IAnnotations
from zope.app.security.settings import Allow, Deny, Unset
from zope.app.security.principal import checkPrincipal

from zope.app.securitypolicy.role import checkRole
from zope.app.securitypolicy.principalrole import \
     AnnotationPrincipalRoleManager, SecurityMap
from zope.app.securitypolicy.interfaces import IPrincipalRoleManager

from sip.security.source import Source, SecurityCache
from zope.app import zapi

class SecurityRoleSource(Source):
    className = 'sip.SecurityRole'
    _keys = ('role_id', 'principal_id')

class SecurityPrincipalSource(Source):
    className = 'sip.SecurityPrincipal'
    _keys = ('principal_id', 'role_id')

securityCache = SecurityCache()

class SQLSecurityMap(SecurityMap):

    def __init__(self, context):
        self.context = context
        self._clear()

    def _clear(self):
        self._rowClass = SecurityRoleSource
        self._colClass = SecurityPrincipalSource

    def _get_byrow(self):
        row = self._rowClass(context=self.context)
        row.__parent__ = self.context
        row.__name__ = 'SecurityRoleSource'
        return row

    _byrow = property(_get_byrow)

    def _get_bycol(self):
        col = self._colClass(context=self.context)
        col.__parent__ = self.context
        col.__name__ = 'SecurityPrincipalSource'
        return col

    _bycol = property(_get_bycol)

    def addCell(self, rowentry, colentry, value):
        row = self._byrow[rowentry]
        row[colentry] = value

        col = self._bycol[colentry]
        col[rowentry] = value

class SQLAnnotationPrincipalRoleManager(SQLSecurityMap):
    """Mappings between principals and roles."""

    implements(IPrincipalRoleManager)

    def assignRoleToPrincipal(self, role_id, principal_id):
        SQLSecurityMap.addCell(self, role_id, principal_id, Allow)

    def removeRoleFromPrincipal(self, role_id, principal_id):
        SQLSecurityMap.addCell(self, role_id, principal_id, Deny)

    unsetRoleForPrincipal = SQLSecurityMap.delCell

    def getSetting(self, role_id, principal_id):
        return SQLSecurityMap.queryCell(
            self, role_id, principal_id, default=Unset)

    getPrincipalsAndRoles = SQLSecurityMap.getAllCells
    # Implementation helpers

    def getRolesForPrincipal(self, principal_id):
        path = zapi.getPath(self.context)
        key = ('principal', principal_id, path)
        result = securityCache.get(key)
        if result is not None:
            return result
        result = SQLSecurityMap.getCol(self, principal_id)
        securityCache[key] = result
        return result

    def getPrincipalsForRole(self, role_id):
        path = zapi.getPath(self.context)
        key = ('role', role_id, path)
        result = securityCache.get(key)
        if result is not None:
            return result
        result = SQLSecurityMap.getRow(self, role_id)
        securityCache[key] = result
        return result

    def _getPrincipalRoles(self, create=0):
        """ Get the principal role map stored in the context, optionally
            creating one if necessary """
        return SQLSecurityMap(self._context)

# Register our cleanup with Testing.CleanUp to make writing unit tests simpler.
from zope.testing.cleanup import addCleanUp
addCleanUp(securityCache.clear)
del addCleanUp
