##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: dashboard.py,v 1.2 2005/10/21 00:15:37 dreamcatcher Exp $
"""

__metaclass__ = type

from zope.app import zapi
from zope.app.publisher.browser import BrowserView
from zope.app.container.interfaces import IReadContainer
from sip.browser import api
from zope.security.proxy import removeSecurityProxy

class Dashboard(BrowserView):

    def __init__(self, context, request):
        self.context = context
        self.request = request
        project = api.getProject(context)
        self.can_add = api.checkSIPPermission(project, 'sip.AddShipment')
        self.can_edit = api.checkSIPPermission(project, 'sip.EditShipment')

    def info(self):
        info = {}
        direction = IReadContainer(self.context)
        funding_factory = api.factory('sip.FundingInfo',
                context = self.context)
        for key, direction in direction.items():
            sub = []
            d_adapter = IReadContainer(direction)
            # To insure states are shown in the right order. May only
            # work with outgoing direction but thats the only one now
            # anyway.
            for k in ['incomplete', 'intransit', 'received']:
                state = d_adapter[k]
                state_adapted = IReadContainer(state)
                shipments = {}
                for sid, shipment in state_adapted.items():
                    naked = removeSecurityProxy(shipment)
                    contact = bool(naked.contact_info_id)
                    shipping = bool(naked.shipping_info_id)
                    funding = funding_factory.selectBy(
                            shipment_id=shipment.id)
                    funding = bool(funding.count())
                    shipments[sid] = {'name':shipment.name,
                                      'contact_complete': contact,
                                      'shipping_complete': shipping,
                                      'funding_complete': funding,
                                      'item':shipment}
                sub.append({'name':state.name,
                           'item':state,
                           'showadd': k == 'incomplete',
                           'count':len(state_adapted),
                           'items':shipments.values()})
            info[key] = {'name':direction.name, 'item':direction,
                         'data':sub}
        return info.values()
