##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: shipment.py,v 1.2 2005/10/21 00:15:37 dreamcatcher Exp $
"""

__metaclass__ = type

from zope.app import zapi
from zope.app.publisher.browser import BrowserView
from zope.app.container.interfaces import IReadContainer
from zope.security.proxy import removeSecurityProxy

from sip.browser import api
from sip.interfaces.shippinginfo import IShippingWizard

class Summary(BrowserView):

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.adapted = IShippingWizard(self.context)
        self.subitems = ['contact_info', 'shipping_info',
                         'funding_info', 'aliquots']
        view = zapi.queryView
        items = [(name, getattr(self.adapted, name, None))
                 for name in self.subitems]
        items = filter(lambda x: x[1] is not None, items)
        items = api.containedItems(items, self.context)
        views = [(name, view(item, 'listing.html', self.request, default=None))
                 for name, item in items]
        views = filter(lambda x: x[1] is not None, views)
        self.views = dict(views)

    def get(self, name, default=None):
        view = self.views.get(name, default)
        if view == default:
            return default
        return {'view':view, 'macro':view['formbody']}

    def pages(self):
        pages = filter(None, [self.get(name) for name in self.subitems])
        return pages

class ShipmentAliquots(BrowserView):

    def __init__(self, context, request):
        self.context = context
        self.request = request
        project = api.getProject(context)
        self.can_edit = api.checkSIPPermission(project, 'sip.EditSample')

    def getAliquotInfo(self):
        extract = self._extractInfo
        return map(extract, self.context.values())

    def _extractInfo(self, aliquot):

        info = {}
        root = api.getRoot(self.context)
        root_url = str(zapi.getView(root, 'absolute_url', self.request))
        sample = removeSecurityProxy(aliquot.sample_)
        info['id'] = aliquot.id
        info['label'] = aliquot.name
        info['sample'] = sample.name
        sample_url = ('%s/projects/sip.Project.%d/samples/sip.Sample.%s' %
                      (root_url, sample.project_id, sample.id))
        info['sample_url'] = sample_url
        info['url'] = '%s?aliquot_id=%s&SAMPLE_EDIT_FORM=1' % (
            sample_url, aliquot.id)
        info['concentration'] = aliquot.getConcentrationString()
        info['weight'] = aliquot.getWeightString()
        info['volume'] = aliquot.getVolumeString()
        info['is_low'] = aliquot.is_low and 'Yes' or 'No'
        if aliquot.aliquot_count is None:
            info['aliquot_count'] = '&nbsp;'
        else:
            info['aliquot_count'] = aliquot.aliquot_count
        info['depleted'] = aliquot.depleted and 'Yes' or 'No'
        return info
