##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: shipment.py,v 1.2 2005/10/21 00:15:20 dreamcatcher Exp $
"""

__metaclass__ = type

from sqlos import SQLOS
from sqlobject import *
from sip.shipment.interfaces import IShipment
from sip.browser import api
from sip.descriptors import FakePropertyDescriptor
from zope.interface import implements
from zope.app.container.contained import Contained

class Shipment(SQLOS, Contained):

    implements(IShipment)

    _idName = 'shipment_id'
    _columns = [
            StringCol('name', length=255, notNull=1),
            DateCol('planneddate', notNull=0),
            DateCol('requestdate', notNull=0),
            DateCol('shipdate', notNull=0),
            DateCol('receivedate', notNull=0),
            DateCol('returndate', notNull=0),
            ForeignKey('ContactInfo', name='contact_info_id', cascade=False),
            ForeignKey('ShippingInfo', name='shipping_info_id', cascade=False),
            ForeignKey('Project', name='project_id', cascade=False),
            BoolCol('infectious', notNull=0, default=False),
            BoolCol('biohazard', notNull=0, default=False),
            StringCol('commit_message', notNull=1),
            ]

    def getRootContainer(self):
        factory = api.factory('sip.Container', context=self)
        query = ('container.container_id = container_shipment.container_id '
                 'and container_shipment.shipment_id = %d '
                 'and container.parent_id = 1' % int(self.id))
        result = factory.select(clause=query, clauseTables=['container_shipment'])
        if result.count():
            return api.contained(result[0], self, name='.')
        return None

    def getRootId(self):
        """returns the container_id for the root container"""
        root = self.getRootContainer()
        return root.id
