##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: utility.py,v 1.2 2005/10/21 00:13:19 dreamcatcher Exp $
"""

__metaclass__ = type

from zope.interface import implements
from zope.app import zapi
from zope.app.container.contained import Contained
from sqlos.interfaces.container import ISQLObjectContainer

from sip.browser import api

class UtilityContainer(Contained):

    implements(ISQLObjectContainer)

    interface = None

    def __init__(self, context=None):
        pass

    def keys(self):
        return [k for k, v in self.items()]

    def __iter__(self):
        return iter(self.keys())

    def values(self):
        for k, v in self.items():
            yield v

    def items(self):
        items = zapi.getUtilitiesFor(self.interface, context=self)
        return [(name, api.contained(value, self, name=name))
                 for name, value in items]

    def __getitem__(self, name):
        item = zapi.queryUtility(self.interface, name=name, context=self)
        if item is None:
            raise KeyError(name)
        return api.contained(item, self, name=name)

    def get(self, name, default=None):
        item = zapi.queryUtility(self.interface, name=name,
                               default=default, context=self)
        if item is default:
            return item
        return api.contained(item, self, name=name)

    def __contains__(self, name):
        return self.get(name, None) is not None

    def __len__(self):
        return len(self.keys())
