##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: units.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""

__metaclass__ = type

from datetime import date
from sqlobject import *
from sqlos import SQLOS
from zope.interface import implements
from zope.security.checker import NamesChecker
from zope.app import zapi

from sip.descriptors import FakePropertyDescriptor
from sip.interfaces.units import IConcentrationUnit, \
     IWeightUnit, IVolumeUnit

class BaseUnit:
    """ Remember Archetypes? :) """

    name = FakePropertyDescriptor('name', 'label')

class ConcentrationUnit(BaseUnit, SQLOS):

    implements(IConcentrationUnit)

    _idName = 'concentration_unit_id'
    _table = 'concentration_unit'

    _columns = [
        StringCol('label', length=255, notNull=1),
        StringCol('description', length=255, notNull=1, default=''),
        DecimalCol('exponent', size=24, precision=18, notNull=1, default=1),
        DateCol('start_date', notNull=1, default=date.today),
        DateCol('end_date', notNull=0, default=None),
        StringCol('commit_message', notNull=1),
        ]


class WeightUnit(BaseUnit, SQLOS):

    implements(IWeightUnit)

    _idName = 'weight_unit_id'
    _table = 'weight_unit'

    _columns = [
        StringCol('label', length=255, notNull=1),
        StringCol('description', length=255, notNull=1, default=''),
        DecimalCol('exponent', size=24, precision=18, notNull=1, default=1),
        DateCol('start_date', notNull=1,default=date.today),
        DateCol('end_date', notNull=0,default=None),
        StringCol('commit_message', notNull=1),
        ]


class VolumeUnit(BaseUnit, SQLOS):

    implements(IVolumeUnit)

    _idName = 'volume_unit_id'
    _table = 'volume_unit'

    _columns = [
        StringCol('label', length=255, notNull=1),
        StringCol('description', length=255, notNull=1, default=''),
        DecimalCol('exponent', size=24, precision=18, notNull=1, default=1),
        DateCol('start_date', notNull=1,default=date.today),
        DateCol('end_date', notNull=0,default=None),
        StringCol('commit_message', notNull=1),
        ]
