##############################################################################
#
# SIP - Sample Inventory Program
# Copyright (C) 2005 RUNYAGA LLC and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: utils.py,v 1.2 2005/10/21 00:08:11 dreamcatcher Exp $
"""

__metaclass__ = type

from zope.app import zapi
from zope.app.container.interfaces import IAdding
from zope.security.management import queryInteraction
from zope.app.datetimeutils import parseDatetimetz
from zope.schema.interfaces import ValidationError
from sip.i18n import _

class UniqueNamesError(ValidationError):
    __doc__ = _("""This value must be unique""")

def getPrincipals():
    principals = []
    interaction = queryInteraction()
    if interaction is not None:
        for participation in interaction.participations:
            principals.append(participation.principal.id)
    return principals

class Unique:

    def __init__(self, className, columnName, limit=0, byProject=False):
        self.className = className
        self.columnName = columnName
        self.limit = limit
        self.byProject = byProject

    def __call__(self, field, value):
        from sqlobject.converters import sqlrepr
        from sip.browser import api
        limit = self.limit
        context = field.context
        if not IAdding.providedBy(field.context):
            limit += 1
        else:
            context = context.context
        if self.byProject:
            project = api.getProject(context)
        context = api.factory(self.className, context=context)
        query = "%s = %s" % (getattr(context.q, self.columnName),
                             sqlrepr(value, 'postgres'))
        if self.byProject:
            query += ' and project_id = %d' % project.id
        items = context.select(query)
        if items.count() > limit:
            raise UniqueNamesError
        return value

class PasswordMatch:

    #def __init__(self, className, columnName):
        #self.className = className
        #self.columnName = columnName

    def __call__(self, field, value):
        form = field.context.request.form
        if value == form['field.password']:
            return value
        return False

def list2query(lst):
    """ converts a list of ints and longs into string suitable for
        queries.
    """

    out = '(%s' % str(lst[0])
    
    for l in lst[1:]:
        out = '%s,%s' % (out, str(l))
    out = '%s)' % out
    return out
